#region File Header
// /////////////////////////////////////////////////////////////
// File: AboutForm.cs	Class: Kennedy.Controls.NumericalTextBox
// Date:10/21/2004		Author: Michael Kennedy
// Language: C#			Framework: .NET
//
// Copyright: Copyright (c) Michael Kennedy, 2004
// /////////////////////////////////////////////////////////////
#endregion

using System;
using System.ComponentModel;

namespace Kennedy.Controls
{
	[DefaultProperty("NumberType")]
	public class NumericalTextBox : ValidatingTextBox
	{
		#region public enum NumericalTypes

		public enum NumericalTypes
		{
			FloatingPoint,
			FloatingPointNonNegative,
			Integer,
			IntegerNonNegative,
			Natural
		}
		#endregion

		#region Member Variables

		private NumericalTypes _numberType = NumericalTypes.FloatingPoint;

		protected string _patternIntegerFull = @"^[\-]?[0-9]+$";
		protected string _patternIntegerPartial = @"^[\-]?[0-9]*$";

		protected string _patternIntegerNonNegFull = @"^[0-9]+$";
		protected string _patternIntegerNonNegPartial = @"^[0-9]*$";

		protected string _patternNaturalFull = @"^[1-9]+[0-9]*$";
		protected string _patternNaturalPartial = @"^[1-9]+[0-9]*$";

		protected string _patternFloatingPointFull = @"^[\-]?[0-9]*\.?[0-9]+$";
		protected string _patternFloatingPointPartial = @"^[\-]?[0-9]*[\.]?[0-9]*$";

		protected string _patternFloatingPointPosFull = @"^[0-9]*\.?[0-9]+$";
		protected string _patternFloatingPointPosPartial = @"^[0-9]*[\.]?[0-9]*$";

		#endregion

		#region Construction

		public NumericalTextBox()
		{
			ConfigurePatterns();
		}
		#endregion

		#region Value and Number Type Related Code

		protected virtual void ConfigurePatterns()
		{
			switch (_numberType)
			{
				case NumericalTypes.FloatingPoint:
					this.PatternPartial = _patternFloatingPointPartial;
					this.PatternFull = _patternFloatingPointFull;
					break;

				case NumericalTypes.FloatingPointNonNegative:
					this.PatternPartial = _patternFloatingPointPosPartial;
					this.PatternFull = _patternFloatingPointPosFull;
					break;

				case NumericalTypes.Integer:
					this.PatternPartial = _patternIntegerPartial;
					this.PatternFull = _patternIntegerFull;
					break;

				case NumericalTypes.Natural:
					this.PatternPartial = _patternNaturalPartial;
					this.PatternFull = _patternNaturalFull;
					break;

				case NumericalTypes.IntegerNonNegative:
					this.PatternPartial = _patternIntegerNonNegPartial;
					this.PatternFull = _patternIntegerNonNegFull;
					break;

				default:
					throw new ArgumentException("Unsupported numerical type: " + _numberType, "NumberType");
			}
		}

		[DefaultValue(NumericalTypes.FloatingPoint)]
		[Description("The type of numbers that are considered valid input.")]
		[Category("Behavior")]
		public NumericalTypes NumberType
		{
			get
			{
				return _numberType;
			}
			set
			{
				_numberType = value;
				ConfigurePatterns();
				RevalidateText();
			}
		}

		[Description("The numerical value of the current text.")]
		[Category("Appearance")]
		public virtual double Value
		{
			get
			{
				if (!IsValid)
				{
					string msg = "The Value property is not accessible when the contents are invalid.";
					throw new InvalidOperationException(msg);
				}

				double val = Convert.ToDouble(this.Text);
				return val;
			}
			set
			{
				string text = value.ToString();
				if (!ValidateString(text, true))
				{
					string msg = "The value specified is invalid based on the current number type.";
					throw new ArgumentException(msg, "Value");
				}
				
				this.Text = text;
			}
		}

		#endregion

	}
}
